-- Create the data warehouse
USE master;
IF EXISTS(SELECT * FROM sys.sysdatabases WHERE name = 'DemoDW')
	DROP DATABASE DemoDW;
GO
CREATE DATABASE [DemoDW]
 ON  PRIMARY 
( NAME = N'DemoDW', FILENAME = N'D:\Demofiles\Mod03\DemoDW.mdf' , SIZE = 5MB , FILEGROWTH = 5MB )
 LOG ON 
( NAME = N'DemoDW_log', FILENAME = N'D:\Demofiles\Mod03\DemoDW.ldf' , SIZE = 1MB , FILEGROWTH = 10%)
GO
ALTER DATABASE DemoDW SET RECOVERY SIMPLE 
GO

USE DemoDW;
GO
CREATE TABLE [dbo].[DimDate](
	[DateKey] [int] NOT NULL,
	[FullDateAlternateKey] [date] NOT NULL,
	[DayNumberOfWeek] [tinyint] NOT NULL,
	[DayNameOfWeek] [nvarchar](10) NOT NULL,
	[DayNumberOfMonth] [tinyint] NOT NULL,
	[MonthName] [nvarchar](10) NOT NULL,
	[MonthNumberOfYear] [tinyint] NOT NULL,
	[CalendarYear] [smallint] NOT NULL,
);

DECLARE @StartDate datetime
DECLARE @EndDate datetime
SET @StartDate = dateadd(year,-2, getdate())
SET @EndDate = dateadd(month, 1, getdate())
DECLARE @LoopDate datetime
SET @LoopDate = @StartDate
WHILE @LoopDate <= @EndDate
BEGIN
  INSERT INTO dbo.DimDate VALUES
	(
		CAST(CONVERT(VARCHAR(8), @LoopDate, 112) AS int) , -- date key
		@LoopDate, -- date alt key
		datepart(dw, @LoopDate), -- day number of week
		datename(dw, @LoopDate), -- day name of week
	    Day(@LoopDate),  -- day number of month
		datename(mm, @LoopDate), -- month name
		Month(@LoopDate), -- month number of year
		Year(@LoopDate) -- calendar year
	)  		  
	SET @LoopDate = DateAdd(dd, 1, @LoopDate)
END;

CREATE CLUSTERED INDEX cids_Dimdate_DateKey ON DimDate(DateKey);
CREATE NONCLUSTERED INDEX idx_DimDate_MonthNumberOfYear ON DimDate(MonthNumberOfYear);
CREATE NONCLUSTERED INDEX idx_DimDate_CalendarYear ON DimDate(CalendarYear);

CREATE TABLE DimCustomer
(CustomerKey integer NOT NULL,
 CustomerAltKey varchar(10) NOT NULL,
 CustomerParentKey integer NULL,
 CustomerName nvarchar(100) NOT NULL,
 CustomerType nvarchar(100) NOT NULL);
INSERT INTO DimCustomer
VALUES (1, '000001', NULL, 'World Wide Importers', 'Retail');
INSERT INTO DimCustomer
VALUES (2, '000002', NULL, 'Margies Travel', 'Travel and Leisure');
INSERT INTO DimCustomer
VALUES (3, '000003', NULL, 'Coho Vineyard and Winery', 'Food and Beverages');
INSERT INTO DimCustomer
VALUES (4, '000004', NULL, 'Contoso Ltd', 'Retail');
INSERT INTO DimCustomer
VALUES (5, '000005', 1, 'Northwind Traders', 'Retail');
INSERT INTO DimCustomer
VALUES (6, '000006', 2, 'Blue Yonder Airlines', 'Travel and Leisure');
INSERT INTO DimCustomer
VALUES (7, '000007', 3, 'Coho Vineyard', 'Food and Beverages');
INSERT INTO DimCustomer
VALUES (8, '000008', 3, 'Coho Winery', 'Food and Beverages');
INSERT INTO DimCustomer
VALUES (9, '000009', 4, 'Fabrikam Inc', 'Retail');
INSERT INTO DimCustomer
VALUES (10, '000010', 2, 'Alpine Ski House', 'Travel and Leisure');

CREATE CLUSTERED INDEX cids_DimCustomer_CustomerKey ON DimCustomer(CustomerKey);
CREATE NONCLUSTERED INDEX idx_DimCustomer_CustomerName ON DimCustomer(CustomerName);
CREATE NONCLUSTERED INDEX idx_DimCustomer_CustomerType ON DimCustomer(CustomerType);

CREATE TABLE DimProduct
(ProductKey integer NOT NULL,
 ProductAltKet integer NOT NULL,
 ProductName nvarchar(20) NOT NULL);
DECLARE @LoopInt integer
SET @LoopInt = 1
WHILE @LoopInt <= 1001
BEGIN
	INSERT INTO DimProduct
	VALUES
	(@LoopInt, @LoopInt, CONCAT('Product ', CONVERT(nvarchar(4), @LoopInt)));
	SET @LoopInt = @LoopInt + 1;
END;

CREATE CLUSTERED INDEX cids_DimProduct_ProductKey ON DimProduct(ProductKey);
CREATE NONCLUSTERED INDEX idx_DimProduct_ProductName ON DimProduct(ProductName);

CREATE TABLE FactOrder
(DateKey integer, CustomerKey integer, ProductKey integer, Quantity integer, SalesAmount money);

INSERT INTO FactOrder
SELECT d.DateKey, c.CustomerKey, p.ProductKey, d.DayNumberOfWeek Quantity, d.DayNumberOfMonth SalesAmount
FROM DimDate d
CROSS JOIN DimCustomer c
CROSS JOIN DimProduct p;

CREATE CLUSTERED INDEX cids_FactOrder_DateKey ON FactOrder(DateKey);
CREATE NONCLUSTERED INDEX cids_FactOrder_CustomerKey ON FactOrder(CustomerKey);
CREATE NONCLUSTERED INDEX cids_FactOrder_ProductKey ON FactOrder(ProductKey);




-- Estimate  size saving
EXECUTE sp_estimate_data_compression_savings 'dbo', 'FactOrder', NULL, NULL, 'PAGE';




-- Create a compressed version of the database
USE master;
IF EXISTS(SELECT * FROM sys.sysdatabases WHERE name = 'CompressedDemoDW')
	DROP DATABASE CompressedDemoDW;
GO
CREATE DATABASE CompressedDemoDW
 ON  PRIMARY 
( NAME = N'CompressedDemoDW_data', FILENAME = N'D:\Demofiles\Mod03\CompressedDemoDW.mdf' , SIZE = 5MB , FILEGROWTH = 5MB )
 LOG ON 
( NAME = N'CompressedDemoDW_log', FILENAME = N'D:\Demofiles\Mod03\CompressedDemoDW.ldf' , SIZE = 1MB , FILEGROWTH = 10%)
GO
ALTER DATABASE CompressedDemoDW SET RECOVERY SIMPLE 
GO

USE CompressedDemoDW;
GO
CREATE TABLE [dbo].[DimDate](
	[DateKey] [int] NOT NULL,
	[FullDateAlternateKey] [date] NOT NULL,
	[DayNumberOfWeek] [tinyint] NOT NULL,
	[DayNameOfWeek] [nvarchar](10) NOT NULL,
	[DayNumberOfMonth] [tinyint] NOT NULL,
	[MonthName] [nvarchar](10) NOT NULL,
	[MonthNumberOfYear] [tinyint] NOT NULL,
	[CalendarYear] [smallint] NOT NULL,
)
WITH (DATA_COMPRESSION = PAGE);

DECLARE @StartDate datetime
DECLARE @EndDate datetime
SET @StartDate = dateadd(year,-2, getdate())
SET @EndDate = dateadd(month, 1, getdate())
DECLARE @LoopDate datetime
SET @LoopDate = @StartDate
WHILE @LoopDate <= @EndDate
BEGIN
  INSERT INTO dbo.DimDate VALUES
	(
		CAST(CONVERT(VARCHAR(8), @LoopDate, 112) AS int) , -- date key
		@LoopDate, -- date alt key
		datepart(dw, @LoopDate), -- day number of week
		datename(dw, @LoopDate), -- day name of week
	    Day(@LoopDate),  -- day number of month
		datename(mm, @LoopDate), -- month name
		Month(@LoopDate), -- month number of year
		Year(@LoopDate) -- calendar year
	)  		  
	SET @LoopDate = DateAdd(dd, 1, @LoopDate)
END;

CREATE CLUSTERED INDEX cids_Dimdate_DateKey ON DimDate(DateKey)
WITH (DATA_COMPRESSION = PAGE);
CREATE NONCLUSTERED INDEX idx_DimDate_MonthNumberOfYear ON DimDate(MonthNumberOfYear)
WITH (DATA_COMPRESSION = PAGE);
CREATE NONCLUSTERED INDEX idx_DimDate_CalendarYear ON DimDate(CalendarYear)
WITH (DATA_COMPRESSION = PAGE);

CREATE TABLE DimCustomer
(CustomerKey integer NOT NULL,
 CustomerAltKey varchar(10) NOT NULL,
 CustomerParentKey integer NULL,
 CustomerName nvarchar(100) NOT NULL,
 CustomerType nvarchar(100) NOT NULL)
 WITH (DATA_COMPRESSION = PAGE);
INSERT INTO DimCustomer
VALUES (1, '000001', NULL, 'World Wide Importers', 'Retail');
INSERT INTO DimCustomer
VALUES (2, '000002', NULL, 'Margies Travel', 'Travel and Leisure');
INSERT INTO DimCustomer
VALUES (3, '000003', NULL, 'Coho Vineyard and Winery', 'Food and Beverages');
INSERT INTO DimCustomer
VALUES (4, '000004', NULL, 'Contoso Ltd', 'Retail');
INSERT INTO DimCustomer
VALUES (5, '000005', 1, 'Northwind Traders', 'Retail');
INSERT INTO DimCustomer
VALUES (6, '000006', 2, 'Blue Yonder Airlines', 'Travel and Leisure');
INSERT INTO DimCustomer
VALUES (7, '000007', 3, 'Coho Vineyard', 'Food and Beverages');
INSERT INTO DimCustomer
VALUES (8, '000008', 3, 'Coho Winery', 'Food and Beverages');
INSERT INTO DimCustomer
VALUES (9, '000009', 4, 'Fabrikam Inc', 'Retail');
INSERT INTO DimCustomer
VALUES (10, '000010', 2, 'Alpine Ski House', 'Travel and Leisure');

CREATE CLUSTERED INDEX cids_DimCustomer_CustomerKey ON DimCustomer(CustomerKey)
WITH (DATA_COMPRESSION = PAGE);
CREATE NONCLUSTERED INDEX idx_DimCustomer_CustomerName ON DimCustomer(CustomerName)
WITH (DATA_COMPRESSION = PAGE);
CREATE NONCLUSTERED INDEX idx_DimCustomer_CustomerType ON DimCustomer(CustomerType)
WITH (DATA_COMPRESSION = PAGE);

CREATE TABLE DimProduct
(ProductKey integer NOT NULL,
 ProductAltKet integer NOT NULL,
 ProductName nvarchar(20) NOT NULL)
 WITH (DATA_COMPRESSION = PAGE);
DECLARE @LoopInt integer
SET @LoopInt = 1
WHILE @LoopInt <= 1001
BEGIN
	INSERT INTO DimProduct
	VALUES
	(@LoopInt, @LoopInt, CONCAT('Product ', CONVERT(nvarchar(4), @LoopInt)));
	SET @LoopInt = @LoopInt + 1;
END;

CREATE CLUSTERED INDEX cids_DimProduct_ProductKey ON DimProduct(ProductKey)
WITH (DATA_COMPRESSION = PAGE);
CREATE NONCLUSTERED INDEX idx_DimProduct_ProductName ON DimProduct(ProductName)
WITH (DATA_COMPRESSION = PAGE);

CREATE TABLE FactOrder
(DateKey integer, CustomerKey integer, ProductKey integer, Quantity integer, SalesAmount money)
WITH (DATA_COMPRESSION = PAGE);
INSERT INTO FactOrder
SELECT d.DateKey, c.CustomerKey, p.ProductKey, d.DayNumberOfWeek Quantity, d.DayNumberOfMonth SalesAmount
FROM DimDate d
CROSS JOIN DimCustomer c
CROSS JOIN DimProduct p;

CREATE CLUSTERED INDEX cids_FactOrder_DateKey ON FactOrder(DateKey)
WITH (DATA_COMPRESSION = PAGE);
CREATE NONCLUSTERED INDEX cids_FactOrder_CustomerKey ON FactOrder(CustomerKey)
WITH (DATA_COMPRESSION = PAGE);
CREATE NONCLUSTERED INDEX cids_FactOrder_ProductKey ON FactOrder(ProductKey)
WITH (DATA_COMPRESSION = PAGE);



-- Compare query performance
USE master;
SET STATISTICS TIME ON;
SET STATISTICS IO ON;
SELECT d.CalendarYear, d.MonthNumberOfYear, c.CustomerType, p.ProductName, SUM(o.Quantity) ItemsSold, SUM(o.SalesAmount) TotalRevenue
FROM DemoDW.dbo.FactOrder o
JOIN DemoDW.dbo.DimDate d ON o.DateKey = d.DateKey
JOIN DemoDW.dbo.DimCustomer c ON o.CustomerKey = c.CustomerKey
JOIN DemoDW.dbo.DimProduct p ON o.ProductKey = p.ProductKey
WHERE d.FullDateAlternateKey BETWEEN (DATEADD(month, -6, getdate())) AND (getdate())
GROUP BY d.CalendarYear, d.MonthNumberOfYear, c.CustomerType, p.ProductName
ORDER BY d.CalendarYear, d.MonthNumberOfYear, c.CustomerType, p.ProductName;

SELECT d.CalendarYear, d.MonthNumberOfYear, c.CustomerType, p.ProductName, SUM(o.Quantity) ItemsSold, SUM(o.SalesAmount) TotalRevenue
FROM CompressedDemoDW.dbo.FactOrder o
JOIN CompressedDemoDW.dbo.DimDate d ON o.DateKey = d.DateKey
JOIN CompressedDemoDW.dbo.DimCustomer c ON o.CustomerKey = c.CustomerKey
JOIN CompressedDemoDW.dbo.DimProduct p ON o.ProductKey = p.ProductKey
WHERE d.FullDateAlternateKey BETWEEN (DATEADD(month, -6, getdate())) AND (getdate())
GROUP BY d.CalendarYear, d.MonthNumberOfYear, c.CustomerType, p.ProductName
ORDER BY d.CalendarYear, d.MonthNumberOfYear, c.CustomerType, p.ProductName;

SET STATISTICS TIME OFF;
SET STATISTICS IO OFF;