USE master;

IF EXISTS(SELECT * FROM sys.sysdatabases WHERE name = 'DemoDW')
	DROP DATABASE DemoDW;
GO

CREATE DATABASE DemoDW;
GO

USE DemoDW;
GO

CREATE TABLE [dbo].[DimDate](
	[DateKey] [int] NOT NULL,
	[FullDateAlternateKey] [date] NOT NULL,
	[DayNumberOfWeek] [tinyint] NOT NULL,
	[DayNameOfWeek] [nvarchar](10) NOT NULL,
	[DayNumberOfMonth] [tinyint] NOT NULL,
	[MonthName] [nvarchar](10) NOT NULL,
	[MonthNumberOfYear] [tinyint] NOT NULL,
	[CalendarYear] [smallint] NOT NULL,
);

-- Populate DimDate dimension table with values from 3 years ago
-- declare variables to hold the start and end date
DECLARE @StartDate datetime
DECLARE @EndDate datetime

--- assign values to the start date and end date we 
-- want our reports to cover (this should also take
-- into account any future reporting needs)
SET @StartDate = dateadd(year,-3, getdate())
SET @EndDate = dateadd(month, 1, getdate())

-- using a while loop increment from the start date 
-- to the end date
DECLARE @LoopDate datetime
SET @LoopDate = @StartDate

WHILE @LoopDate <= @EndDate
BEGIN
 -- add a record into the date dimension table for this date
  INSERT INTO dbo.DimDate VALUES
	(
		CAST(CONVERT(VARCHAR(8), @LoopDate, 112) AS int) , -- date key
		@LoopDate, -- date alt key
		datepart(dw, @LoopDate), -- day number of week
		datename(dw, @LoopDate), -- day name of week
	    Day(@LoopDate),  -- day number of month
		datename(mm, @LoopDate), -- month name
		Month(@LoopDate), -- month number of year
		Year(@LoopDate) -- calendar year
	)  		  
	 -- increment the date by 1 day and do next loop
	SET @LoopDate = DateAdd(dd, 1, @LoopDate)
END

CREATE TABLE DimProduct
(ProductKey integer NOT NULL,
 ProductAltKey varchar(10) NOT NULL,
 ProductName nvarchar(100) NOT NULL,
 Category nvarchar(100) NOT NULL);

DECLARE @i integer = 1;
WHILE @i < 12
BEGIN
	INSERT INTO DimProduct
	VALUES (@i, replace(str(@i, 6), ' ', '0'), 'Product ' + CONVERT(nvarchar(3), @i), 'Category ' + CONVERT(nvarchar(3), ((@i % 3) + 1)) );
	SET @i = @i + 1;
END;

CREATE TABLE DimCustomer
(CustomerKey integer NOT NULL,
 CustomerAltKey varchar(10) NOT NULL,
 CustomerName nvarchar(100) NOT NULL,
 CustomerType nvarchar(100) NOT NULL,
 City nvarchar(100) NOT NULL,
 Country nvarchar(100) NOT NULL);

INSERT INTO DimCustomer
VALUES (1, '000001', 'World Wide Importers', 'Retail', 'London', 'United Kingdom');

INSERT INTO DimCustomer
VALUES (2, '000002', 'Margies Travel', 'Travel and Leisure', 'San Francisco', 'United States');

INSERT INTO DimCustomer
VALUES (3, '000003', 'Coho Vineyard and Winery', 'Food and Beverages', 'Bothell', 'United States');

INSERT INTO DimCustomer
VALUES (4, '000004', 'Contoso Ltd', 'Retail', 'Paris', 'France');

INSERT INTO DimCustomer
VALUES (5, '000005', 'Northwind Traders', 'Retail', 'Cairns', 'Australia');

INSERT INTO DimCustomer
VALUES (6, '000006', 'Blue Yonder Airlines', 'Travel and Leisure', 'San Francisco', 'United States');

INSERT INTO DimCustomer
VALUES (7, '000007', 'Fabrikam Inc', 'Retail', 'Dunfermline', 'United Kingdom');

INSERT INTO DimCustomer
VALUES (8, '00008', 'Alpine Ski House', 'Travel and Leisure', 'Tignes', 'France');

INSERT INTO DimCustomer
VALUES (9, '00009', 'Tailspin Toys', 'Retail', 'Sydney', 'Australia');

INSERT INTO DimCustomer
VALUES (10, '00010', 'Woodgrove Bank', 'Financial Services', 'London', 'United Kingdom');

INSERT INTO DimCustomer
VALUES (11, '00011', 'Wingtip Toys', 'Retail', 'New York', 'United States');




CREATE TABLE FactSalesOrder
(OrderDateKey integer NOT NULL,
 ShipDateKey integer NOT NULL,
 ProductKey integer NOT NULL,
 CustomerKey integer NOT NULL,
 OrderNumber integer NOT NULL,
 Cost money NOT NULL,
 SalesAmount money NOT NULL,
 OrderDate date,
 ShipDate date);

SET @LoopDate = @StartDate
DECLARE @OrderNo AS integer = 999;
WHILE @LoopDate <= dateadd(d, -1, getdate())
BEGIN
	SET @i = 1
	DECLARE @NumOrders AS Integer;
	SET @NumOrders = (ROUND(RAND() * 10, 0)) + 1;
	WHILE @i < @NumOrders
	BEGIN
		DECLARE @od as int;
		SELECT @od = CAST(CONVERT(VARCHAR(8), @LoopDate, 112) AS int);
		DECLARE @sd as int;
		SELECT @sd = CAST(CONVERT(VARCHAR(8), DATEADD(d, 2,@LoopDate), 112) AS int);
		DECLARE @p as int;
		SELECT @p = (ROUND(RAND() * 10, 0)) + 1;
		DECLARE @c as int;
		SELECT @c = (ROUND(RAND() * 10, 0)) + 1;
		DECLARE @s as money;
		SELECT @s = ((@p * 99) + (((YEAR(@LoopDate) - YEAR(@StartDate)) + 0.1) * 1.1));
		DECLARE @ct as money;
		SELECT @ct = ((@p % 3) + 5) * 55 ;
		SET @OrderNo = @OrderNo + 1
		INSERT INTO FactSalesOrder VALUES (@od, @sd, @p, @c, @OrderNo, @ct, @s, DATEADD(d, -1, @LoopDate), @LoopDate);
		SET @i = @i + 1;
	END
SET @LoopDate = dateadd(d, 1, @LoopDate)
END;

SELECT d.CalendarYear, c.Country, p.Category, SUM(f.SalesAmount) Revenue, SUM(f.Cost) Cost, ((SUM(f.SalesAmount) - SUM(f.Cost)) / SUM(f.SalesAmount)) * 100 Margin
FROM FactSalesOrder f
JOIN DimDate d ON f.OrderDateKey = d.DateKey
JOIN DimCustomer c ON f.CustomerKey = c.CustomerKey
JOIN DimProduct p ON f.ProductKey = p.ProductKey
GROUP by d.CalendarYear, c.Country, p.Category
ORDER BY d.CalendarYear, c.Country, p.Category