-- View filegroups and partitions
USE DemoDW;
SELECT OBJECT_NAME(p.object_id) as obj_name, f.name, p.partition_number, p.rows
FROM sys.system_internals_allocation_units a
JOIN sys.partitions p
ON p.partition_id = a.container_id
JOIN sys.filegroups f ON a.filegroup_id = f.data_space_id
WHERE p.object_id IN (OBJECT_ID (N'dbo.fact_table'), OBJECT_ID (N'dbo.dim_table'))
ORDER BY obj_name, p.index_id, p.partition_number;
GO

-- Make inactive filegroups read-only
ALTER DATABASE DemoDW
MODIFY FILEGROUP FG0000 READONLY;
ALTER DATABASE DemoDW
MODIFY FILEGROUP FG2000 READONLY;
ALTER DATABASE DemoDW
MODIFY FILEGROUP FG2001 READONLY;
ALTER DATABASE DemoDW
MODIFY FILEGROUP FG2002 READONLY


-- Backup inactive filegroups
BACKUP DATABASE DemoDW
FILEGROUP = 'FG0000'
TO DISK = 'D:\Demofiles\Mod11\DemoDWFG0000.bak'
WITH FORMAT, INIT, COMPRESSION;

BACKUP DATABASE DemoDW
FILEGROUP = 'FG2000'
TO DISK = 'D:\Demofiles\Mod11\DemoDWFG2000.bak'
WITH FORMAT, INIT, COMPRESSION;

BACKUP DATABASE DemoDW
FILEGROUP = 'FG2001'
TO DISK = 'D:\Demofiles\Mod11\DemoDWFG2001.bak'
WITH FORMAT, INIT, COMPRESSION;

BACKUP DATABASE DemoDW
FILEGROUP = 'FG2002'
TO DISK = 'D:\Demofiles\Mod11\DemoDWFG2002.bak'
WITH FORMAT, INIT, COMPRESSION;

-- Backup read/write filegroups
USE master;
BACKUP DATABASE DemoDW
READ_WRITE_FILEGROUPS
TO DISK = 'D:\Demofiles\Mod11\DemoDWRWFull.bak'
WITH FORMAT, INIT, COMPRESSION;

-- Perform data load
USE DemoDW;
INSERT dim_table
VALUES
('New value');

CREATE TABLE staging_table
 (datekey int, measure int)
ON FG2003;

ALTER TABLE staging_table
WITH CHECK ADD CONSTRAINT bounds
check (datekey >= 20030101 and datekey < 20040101 and datekey IS NOT NULL);

INSERT staging_table VALUES (20030101, 100);
INSERT staging_table VALUES (20030102, 100);

ALTER DATABASE DemoDW ADD FILEGROUP FG2004;

ALTER DATABASE DemoDW ADD FILE (NAME = F2004, FILENAME = 'D:\Demofiles\Mod11\F2004.ndf', SIZE = 3MB, FILEGROWTH = 50%) TO FILEGROUP FG2004;

ALTER PARTITION SCHEME PS
NEXT USED FG2004;

ALTER PARTITION FUNCTION PF() SPLIT RANGE(20040101);

DECLARE @p int = $PARTITION.PF(20030101);
ALTER TABLE staging_table
SWITCH TO fact_table PARTITION @p;

DROP TABLE staging_table;

-- Make loaded filegroup read-only and back it up
USE master;
ALTER DATABASE DemoDW
MODIFY FILEGROUP FG2003 READONLY;

BACKUP DATABASE DemoDW
FILEGROUP = 'FG2003'
TO DISK = 'D:\Demofiles\Mod11\DemoDWFG2003.bak'
WITH FORMAT, INIT, COMPRESSION;

-- Perform a differential backup of read/write filegroups
BACKUP DATABASE DemoDW
READ_WRITE_FILEGROUPS
TO DISK = 'D:\Demofiles\Mod11\DemoDWRWDiff.bak'
WITH DIFFERENTIAL, FORMAT, INIT, COMPRESSION;

-- Simulate a disaster
DROP DATABASE DemoDW;

-- Restore the initial full backup with the partial option
RESTORE DATABASE DemoDW
FROM DISK = 'D:\Demofiles\Mod11\DemoDWRWFull.bak'
WITH PARTIAL, NORECOVERY
GO

-- Restore the read/write filegroups and recover
RESTORE DATABASE DemoDW
FROM DISK = 'D:\Demofiles\Mod11\DemoDWRWDiff.bak'
WITH RECOVERY
GO

-- Access read/write data
SELECT * FROM DemoDW.dbo.dim_table;

-- Restore the read-only filegroups
RESTORE DATABASE DemoDW
FILEGROUP = 'FG0000'
FROM DISK = 'D:\Demofiles\Mod11\DemoDWFG0000.bak'
WITH RECOVERY;

RESTORE DATABASE DemoDW
FILEGROUP = 'FG2000'
FROM DISK = 'D:\Demofiles\Mod11\DemoDWFG2000.bak'
WITH RECOVERY;

RESTORE DATABASE DemoDW
FILEGROUP = 'FG2001'
FROM DISK = 'D:\Demofiles\Mod11\DemoDWFG2001.bak'
WITH RECOVERY;

RESTORE DATABASE DemoDW
FILEGROUP = 'FG2002'
FROM DISK = 'D:\Demofiles\Mod11\DemoDWFG2002.bak'
WITH RECOVERY;

RESTORE DATABASE DemoDW
FILEGROUP = 'FG2003'
FROM DISK = 'D:\Demofiles\Mod11\DemoDWFG2003.bak'
WITH RECOVERY;


-- Access read-only data
SELECT * FROM DemoDW.dbo.fact_table;